@@ -1,4 +1,4 @@
-context("site")
+local_edition(3)
 
 # copy part of our demo site to a tempdir
 local_create_site <- function(files, env = parent.frame()) {
@@ -13,7 +13,7 @@ local_create_site <- function(files, env = parent.frame()) {
   site_dir
 }
 
-test_that("render_site", {
+test_that("render_site works", {
 
   # copy our demo site to a tempdir
   site_dir <- local_create_site()
@@ -41,6 +41,18 @@ test_that("render_site", {
   expect_true(all(!file.exists(file.path(site_dir, "_site", excluded))))
 })
 
+test_that("site_generator() correctly try to use custom site generator", {
+  # copy our demo site to a tempdir
+  site_dir <- local_create_site()
+  withr::local_dir(site_dir)
+  expect_equal(site_generator(), default_site("."))
+  xfun::gsub_file("index.Rmd", "(output: html_document)", "\\1\nsite: rmarkdown::dummy_site")
+  # It errors if not found
+  expect_error(site_generator(), regexp = "'dummy_site' is not an exported object from 'namespace:rmarkdown'", fixed = TRUE)
+  file.rename("index.Rmd", "index.rmd")
+  expect_error(site_generator(), regexp = "'dummy_site' is not an exported object from 'namespace:rmarkdown'", fixed = TRUE)
+})
+
 test_that("render_site respects 'new_session' in the config", {
 
   skip_if_not_installed("xfun", "0.13")
